<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');
/**-------------------------------------------------------------------------
 *
 *	Auth Controller
 *	Process users login, regiration, profile, logout
 *
 * @file:  		./application/controllers/install.php
 * @version:	0.9.3
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/

class Auth extends Mauth_Controller
{
	// private page data	
	var $data;
	

	/**
	 * Constructor
	 */
	function __construct()
	{
		parent::__construct();
		
		$this->container = 'container';		
		
		$this->data['userinfo'] = $this->session->userdata('user_data');
	}
	
	/*
    |---------------------------------------------------------------------
	| Register an account   (step 1)
    |---------------------------------------------------------------------
	| upon sucessful registration parts (username, password submitted)
	| displached to _doregister() function.
	|
	*/	
	function register() {
		
		$this->_checkloggedin();
		
		$this->load->library('form_validation');
		
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
		
		$this->form_validation->set_rules(
				'username', 
				'Username', 
				'required'.
					(isset($this->mauth->config['min_username_length']) ? '|min_length['.$this->mauth->config['min_username_length'] .']' : '') .
					(isset($this->mauth->config['max_username_length']) ? '|max_length['.$this->mauth->config['max_username_length'] .']' : '') .
					'|callback_checkusername'
				);
		$this->form_validation->set_rules('email', 'Email', 'required|valid_email|callback_check_email');
		
		
		if ($this->form_validation->run() == FALSE) {
		
			if($this->session->userdata('strikes') >= $this->mauth->config['max_login_strikes'])
				$this->data['disabled'] = TRUE;
			
			$this->template->write('title', 'Registration');
			
			$this->template->write_view('content', $this->mauth->config['template_folder'] .'register/first', $this->data, true);
		
			$this->display();
		
		} 
		else
		{
			$this->_doregister($this->input->post('username'), $this->input->post('email'));
		}
	}
	
	
	/*
    |---------------------------------------------------------------------
	| Confirm Registration  (step 2)
    |---------------------------------------------------------------------
    | If a user comes accross this URL, they will need a reg code.
	|
	|	@param:  RegCode
	*/
	function confirm_registration()
	{
		$this->_checkloggedin();
		
		$regcode = $this->uri->segment(3);
		
		if(empty($regcode)) { 
			redirect( register_url() ); 
		}
			
		$user = $this->mauth->confirmRegistration($regcode);
		
		if(!$user) {
			redirect($this->mauth->config['reg_url']);
		}
	
		// Fix a redirect loop!!
		$this->session->set_flashdata('referrer', settings_url());
	
		$this->load->library('form_validation');
		
		$this->form_validation->set_error_delimiters('<span class="error">', '</span>');
		
		$this->form_validation->set_rules(
				'username', 
				'Username', 
				'required'.
					(isset($this->mauth->config['min_username_length']) ? '|min_length['.$this->mauth->config['min_username_length'] .']' : '') .
					(isset($this->mauth->config['max_username_length']) ? '|max_length['.$this->mauth->config['max_username_length'] .']' : '') .
					'|callback_checkusername'
				);
		$this->form_validation->set_rules('password', 'Password', 'trim|required|matches[passconf]|callback_generate_password');
		$this->form_validation->set_rules('passconf', 'Password Confirmation', 'trim|required');	
		$this->form_validation->set_rules('email', 'Email', 'trim|required|callback_check_email');
		
		if ($this->form_validation->run() == FALSE)
		{

			$this->template->write_view('content',$this->mauth->config['template_folder'] .'register/complete', $user, true);

			$this->display();

		}
		else
		{		
			// complete registration
				
			$user = array(
						'username'  => $this->input->post('username'),
						'password'  => set_value('password'),
						'email' 	=> $this->input->post('email')
						 );
					 
				
			$this->mauth->delete_temp_user($regcode);
				
			$this->users_model->add_user($user);
				
			redirect( login_url() );	
		}
	}
	
	
	/*
    |---------------------------------------------------------------------
	| Denied URL
    |---------------------------------------------------------------------	
    | This url is shown when access is restricted for a logged in user.
	|
	*/
	function denied()
	{
		$this->template->write('title', 'Access Denied', true);
		
		$this->template->write_view('content', $this->mauth->config['template_folder'] .'denied', $this->data, true);
	
		$this->display();
	}
	

	/*
    |---------------------------------------------------------------------
	| Login page
    |---------------------------------------------------------------------
    | Displays a login form and validates user credentials on a http POST
    |
	*/
	function login()
	{	
		$this->_checkloggedin();	

		$this->navigation->chew('auth');
		$this->navigation->crumb( $this->mauth->config['login_url'], 'Login to '. $this->mauth->config['title']);

		// Fix a redirect loop!!
		$this->session->keep_flashdata('referrer');

		$referrer = $this->session->flashdata('referrer');
		$seg = explode('/', $referrer);

		foreach( $seg as $s ) {
			if($seg == 'auth') {
				$referrer = '';
				break;
			}
		}
		
		$this->load->library('form_validation');
		
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
		
		$this->form_validation->set_rules('username', 'Username', 'required');
		$this->form_validation->set_rules('password', 'Password', 'required|callback_process_login');
				
		if ($this->form_validation->run() === FALSE)
		{
			$this->_displayloginform();
		}
		else
		{
			if($this->input->post('cookieuser') == 'do_cookie_user')
			{	
				$this->mauth->create_cookie_user();
			}
		
			// set to redirect to page requested.
			redirect($referrer);
		}
	}
	
	/*
    |---------------------------------------------------------------------
	| Handles a logout
    |---------------------------------------------------------------------
    | Destroys a users session by logging out in db session, and removing 
    | the `remember me` cookie
    |
	*/
	function logout()
	{	
		$referrer = $this->mauth->referrer;
		
		$this->mauth->logout();

		// redirect to previous page...
		redirect($referrer);
	}
	

	/*
    |---------------------------------------------------------------------
	| Process Login
    |---------------------------------------------------------------------
    | Handles a posted user login and returns true or false
    |
	*/
	function process_login() 
	{	
		$username = $this->input->post('username');
		$password = $this->input->post('password');

		if ($userid = $this->mauth->login($username, $password))
		{	
			return TRUE;
		}
		else
		{
			// credentials are bad
			$this->form_validation->set_message('process_login','The credentials you have provided seem to be incorrect.');
		
			return FALSE;
		}
	}
	

	/** 
	 *	Display Login Form & Registration Form
     * 
     * @access:      private 
     * @post:        a login form is displayed to the page.
	 */
	function _displayloginform()
	{

		if($this->session->userdata('strikes') >= 5) {
			$this->data['disabled'] = true;
		}
				
		$pagedata['loginform'] = $this->load->view( $this->mauth->config['template_folder'].'login_form.php', $this->data, TRUE );
		
		$this->template->write_view('content', $this->mauth->config['template_folder'] .'login', $pagedata, TRUE );

		$this->display();
	}
	
	
	/** 
	 *	Callback Generate Password
	 *	a callback and wrapper function for generating a password
     * 
     *  @access:      private
	 */
	function generate_password($password)
	{
		if(empty($password)) return false;
			
		$hash = $this->mauth->make_password($password);
		
		
		return $hash;
	}
	
	/** 
	 *  Callback Username Check
	 *	Makes sure we have a unique username on registration
	 *
     *  @access:    private
	 *	@params:    string      username, required
	 *  @return:	boolean     
	 */
	function check_username($username)
	{
		
		$isvalid = true;
		
		if($this->users_model->get_user($username))
		{
			$isvalid = false;
			$this->form_validation->set_message('check_username', "Someone has already registered with this %s");
		}
			
		return $isvalid;
	
	}
	
	/** 
	 *	Callback Email Check
	 *	make user the email is unique if config says we require it.
	 * 
     *  @access:    private
     *  @param:     string      email address
	 *	@return:    boolean
	 */
	function check_email($email)
	{
	
		$isvalid = true;
		
		if(!$this->mauth->config['unique_email']) return $isvalid;
		
		if($this->users_model->get_user($email)) {
			$isvalid = false;
			$this->form_validation->set_message('check_email', "Someone has already registered with this %s");
		}
		return $isvalid;
	}
	
	
	/** 
	 *	Do Register
	 *	this allows us to complete step 1 of the registration process
	 *  an email is dispatched to the email address provided.  this validates the 
	 *  user registration, we then ask for a password.
     *  
     *  @access     private
     *  @param      string      desired username
     *  @param      string      email address
	 */
	function _doregister($username, $email)
	{
		
		$this->load->library('email');
		
		$config = $this->config->item('mauth');
		
		$econfig['mailtype'] = 'html';
		$econfig['priority'] = '5';
		
		$this->email->initialize($econfig);
		
		$this->email->from($config['reg_from_email'] , $this->mauth->config['title']);
		$this->email->to($email);

		$this->email->subject( $this->mauth->config['title'] .' >> '. $username .', Your account requires activation');
		
		
		// build account into temp registration table.
		$regcode = $this->mauth->register($username, $email);
		
		
		$message['username'] = $username;
		$message['email'] = $email;
		$message['regkey'] = $regcode;
		
		$email = $this->load->view($this->mauth->config['template_folder'] .'/email/register', $message, true);
		
		
		$this->email->message($email);

		$this->email->send();

		$this->template->write_view('content',$this->mauth->config['template_folder'] .'register/thanks', $message, true);
		
		$this->display();
		
	}
	
	/** 
	 *	Logged In Check
	 *  checks if a user is already logged in, if so, redirect to previous page.
     * 
     *  @access     private
	 */
	function _checkloggedin()
	{
		if( $this->session->userdata('logged_in') )
		{
			redirect($this->mauth->referrer);
		}
	}
}

/*	end of file:	auth.php
	file location:	./application/controllers/auth.php		*/
