<?PHP if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	Mauth Profile ***TEMPLATE***
 *	
 *  This is a sample controller for a user profile.  You should can easily
 *  extend the user capabilities here.  
 *
 * @file:  		./application/controllers/profile.php
 * @version:	0.9.2
 * @modified:	March 16, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/

class profile extends Mauth_Controller {

	
	public function __construct() {
		
		parent::__construct();
	
		$this->load->helper('gravatar');
	}
	
	public function index() {
	
		if( $this->session->userdata('logged_in') )
		{
			$username = $this->session->userdata('username');
			redirect( profile_url($username), 301 );
		}
		else
		{
			redirect( register_url() ); // cannot view own profile.. maybe register?
		}
	}
	
	public function view() {

		$username = $this->uri->segment(3); // set to rsegment if you do custom routes
		
		if( empty($username) ) {
			show_404();
			exit;
		}

		$this->load->helper('date');

		// remove the breadcrumbs,  they aren't needed.
		$this->navigation->eat();
		$this->navigation->crumb( array(
				'' => 'Home',
				'users' => 'Users',
				$this->uri->uri_string() => 'Viewing profile'
		));

		$profile = $this->users_model->get_profile($username);
		$profile['avatar'] = get_gravatar( $profile['email'], 120, 'mm');
		
		
		$profile['joined'] = timespan( mysql_to_unix($profile['joined']), time() );

		$this->template->write_view('content', $this->config->item('template_folder', 'mauth') .'profile/view', $profile);
		
		$this->display();
	}
	
	/**
	 *	Settings
	 *	profile settings,  used for settings like privacy, username change, ect...
	 */
	public function settings() {
		
		if( ! $this->session->userdata('logged_in') )
		{
			// not logged in
			redirect( login_url() );
		}

		$this->edit( $this->session->userdata('username') );
	}

	/**
 	 *	Edit profile
 	 */
	public function edit( $username = '' ) {

		if( empty($username) )
		{
			$username = $this->uri->segment(3);
		}

		// restrict editing to perfered users only.
		if( ! $this->permissions->CAN_edit_profiles && $username != $this->session->userdata('username') )
		{
			redirect( denied_url() );
		}

		$this->load->helper('form');
		
		$this->navigation->chew('profile');

		$profile = $this->users_model->get_profile($username);
		$profile['avatar'] = get_gravatar( $profile['email'], 120 );
		
		if( $this->permissions->CAN_edit_user && $this->permissions->CAN_admin ) {
			$profile['roles'] = array();

			$roles = $this->permissions->get_roles();

			foreach($roles as $role) {
				$profile['roles'][$role['roleid']] = $role['name'];
			} 
		}

		$this->template->write_view('content', 'Mauth/profile/settings', $profile);
		
		$this->display();
	}
}

/*	end of file:	profile.php
	file location:	./application/controllers/profile.php	*/
