<?php if (!defined('BASEPATH')) exit('No direct script access allowed'); 

/**-------------------------------------------------------------------------
 *
 *	Mauth Facebook Helper
 *	includes functions useful for facebook integration
 *
 * @file:  		application/third_party/mauth/helpers/mauth_helper.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/
 

/**
 *	Login URL
 *	uses config[mauth][login_url]
 *
 *	@return siteurl($config[mauth][login_url]);
 */
 function login_url() {
 
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');

	return site_url($config['login_url']);
 }
 
 
 /**
 *	Logout URL
 *	uses config[mauth][login_url]
 *
 *	@return siteurl($config[mauth][login_url]);
 */
 function logout_url() {
 
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');

	return site_url($config['logout_url']);
 }
 
 /**
  *	Denied URL 
  * Prints uri to denied location set in Mauth Config.
  *
  *	@return site_url($config[mauth][denied_url] . '/'. $regkey);
  */
 function denied_url() {
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');
 	
 	return site_url($config['denied_url']);
 } 
 
 
 /**
  *	Profile URL 
  * Prints a profile url set in Mauth Config.
  *
  *	@return site_url($config[mauth][denied_url] . '/'. $regkey);
  */
 function profile_url($username = '') {
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');
 	
 	return site_url($config['profile_url'] .'/'. $username);
 } 

 /**
  *	Profile URL 
  * Prints a profile url set in Mauth Config.
  *
  *	@return site_url($config[mauth][denied_url] . '/'. $regkey);
  */
 function settings_url() {
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');
 	
 	return site_url($config['settings_url']);
 } 
 
 /**
  *	Registration URL 
  *	prints a registration url using $config[mauth][reg_url]
  *
  *
  *	@return site_url($config[mauth][reg_url]);
  */
 function register_url() {
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');
 	
 	return site_url($config['reg_url']);
 }
 
 
 /**
  *	Registration URL 
  *	prints a complete registration url using $config[mauth][comp_reg_url]
  *
  *
  *	@return site_url($config[mauth][reg_url] . '/'. $regkey);
  */
 function complete_reg_url($regkey = '') {
 	$ci = get_instance();
 	
 	$config = $ci->config->item('mauth');
 	
 	return site_url($config['comp_reg_url']) . '/'. $regkey;
 }
 
 /** 
  *	Is Facebook Connect configued
  *
  *	@return	bool	indicating if facebook connect is configured
  *					an error may be shown if the facebook library is not loaded
  *					but the facebook integration is.
  */
 function is_fbconnect_enabled() {
 	$ci = get_instance();
	$config = $ci->config->item('mauth');
	
	return $config['facebook']['enabled'];
 }

/*	end of file:	mauth_helper.php
	file location:	./application/helpers/mauth_helper.php	*/
