<?php if (!defined('BASEPATH')) exit('No direct script access allowed'); 

/**-------------------------------------------------------------------------
 *
 *	Mauth Library
 *
 * @file:  		application/third_party/mauth/libraries/mauth.php
 * @version:	0.9.2
 * @modified:	October, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/

class Mauth
{

	// ci repository
	private $ci;
	
	// mauth config vars
	public $config;
    private $session_cookie_name = 'mauth_sess';
    
	
	// session store
	public $session_id;
	public $fb_session_id;
	
	// server recognition
	public $userid;
	public $fbuserid;
	private $session_secret;
	 
	// remote user info
	public $ip_address;
	public $user_agent;
	public $referrer;
	public $location;


	/** 
	 *	Mauth Constructor
	 *	pre-loads ci repository
	 *
	 *	creates a user session.
	 */
	public function __construct()
	{
		
		log_message('debug', 'MAUTH:	Mauth Library Loaded');	
	
    	$this->ci =& get_instance();
		
		$this->activity = time();
		
		$this->config = &$this->ci->config->item('mauth');
		
		$this->_check_version();

        if( ! isset( $this->ci->session ) )
            return;

        // Validate the session credentials if not logged in
        if( ! $this->ci->session->userdata('logged_in') )
        {
		    $this->ci->session->set_userdata( array(
				    'username'	=> 'guest',
				    'userid'	=> 0,
				    'fbuserid'	=> 0,
				    'roleid'	=> 0,
				    'logged_in'	=> FALSE,
				    'email'		=> '',
				    'strikes'	=> 0,
				    'avatar'	=> '',
			        'fb_user'	=> FALSE
			    )
            );

            $this->_authenticate();
		}

        // set path info
		$this->referrer = $this->ci->session->flashdata('referrer');
		$this->location = $this->ci->uri->uri_string();
        $this->ci->session->set_flashdata('referrer', $this->location);
	}

	
	/** 
	 * Authenticates a users session and determines whether they are a valid
     * mauth user.
	 */
	private function _authenticate()
	{		
        // load session information
        $this->session_id = $this->ci->session->userdata('session_id');
        $this->fb_session_id = $this->ci->session->userdata('fb_session_id');

		// if connecting w/ facebook, check for FB session
		if(is_fbconnect_enabled()) 
		{
			$this->fbuserid = $this->ci->facebook->get_loggedin_user();
		}
		
		if ($this->fbuserid)
		{	
			//	Try to create a facebook session, is it valid?
			try
			{
                // TODO:  make sure this is correct
				$fbsession = $this->ci->facebook->do_get_session($_GET['auth_token']);
				$fb_session_id = $this->ci->facebook->promote_session();
			
                if( $this->fb_session_id == $fb_session_id )
                {
					// check if valid user?
                    $this->ci->session->set_userdata('fb_session_id', $this->fb_session_id);
                    $this->ci->session->set_userdata('logged_in', TRUE );
				}
				else
				{	
                    $this->ci->load->add_package_path(APPPATH.'third_party/mauth/');
                    $this->ci->load->model('user_model'); // MOVE TO users_model

					// .. if no session, check if the fbuser is registered				
					$fbuser = $this->ci->users_model->get_user($this->fbuserid, TRUE);
		
					if($fbuser)
					{
                        // LOGGIN IN as FB USER
						$this->ci->session->set_userdata($fbuser);  
						$this->ci->session->set_userdata('logged_in', TRUE);
						$this->ci->session->set_userdata('fb_user', TRUE);						
					}
					else
					{
						// no user associated with FB_USERID
						// check if we are logged in via session...
                        $this->userid = $this->ci->session->userdata('userid');
						if($this->userid != 0 && $this->ci->session->userdata('logged_in'))
						{					
							log_message('debug',"MAUTH:\merging facebook and local accounts for later use.");
						
							$this->ci->session->set_userdata('fbuserid', $this->fbuserid);
							
                            //
                            // TODO: MOVE THIS TO users_model
                            // $this->ci->users_model->assoc_fb_user($this->fbuserid, $userid);
                            //
							$sql = "UPDATE ". $this->config['table_prefix'] ."users
								    SET fbuserid=". $this->fbuserid."
								    WHERE userid=". $this->user['userid'];
							
							$this->ci->db->query($sql);
							
						}
						else
						{
							// we don't have a valid facebook account... hooking time.
                            // this would probably save some flash data and redirect to a 'special'
                            // registration page to properly associate a user with their facebook
                            // account.
							$this->ci->hooks->call('fb_login_no_account', 'call');
						}
                        $this->ci->hooks->call('post_fb_login', 'call');				
					}
				}
			}
			catch(Exception $e)
			{
                // reset user to logged out status.
				$this->ci->session->set_userdata( array(
					'username'	=> 'guest',
					'userid'	=> 0,
					'fbuserid'	=> 0,
					'roleid'	=> 0,
					'logged_in'	=> FALSE,
					'email'		=> '',
					'strikes'	=> 0,
					'avatar'	=> '',
					'permission'=> '')
                );			
			}
		}
		else	// not logged in via facebook
		{		
            if($this->check_cookie_user())
    	        log_message('debug', "MAUTH:	Cookie login completed.\n\n");
		}
	}

	
	/**
	 *  Checks user & password against stored user
	 * 
	 *  @access	public
	 *  @param	string	username
	 *  @param	string	password
	 *  @return	bool
	 */
	public function login($username, $password)
	{	
		if(!empty($username) && !empty($password))
		{	
    		$user = $this->ci->users_model->get_user($username);
			
			if($user)
			{	
                $strikes = $this->ci->session->userdata('strikes');
				if($this->_check_password($password, $user['password']) && $strikes < 5)
				{			
                    log_message('debug', "MAUTH:\tuser successfully logged in.");	
					$this->ci->session->set_userdata( $user );
					$this->ci->session->set_userdata( 'logged_in', TRUE );
					$this->strikes = 0; // reset strikes

                    //TODO:  add hook point.
					$this->ci->hooks->call('post_login', 'call');

					return TRUE;
				}
				else
				{
                    $strikes = $this->ci->session->userdata('strikes');
					$this->ci->session->set_userdata('strikes',  $strikes + 1);
				}
			}
            log_message('debug', "MAUTH:\tuser not found.");
		}
		return FALSE;
	}
	

	/** 
	 *  Creates a cookie for the user.
	 *
	 *  @access	public
	 *  @return	none
	 */
	function create_cookie_user()
	{
        $this->ci->load->helper('cookie');

		$cookievalue = array(
			'username' => $this->ci->session->userdata('username'),
			'password' => $this->ci->session->userdata('password'),
		);
		
		$cookie = array(
                	'name'   => $this->session_cookie_name,
               		'value'  => serialize($cookievalue),
                	'expire' => $this->config['cookie_expires'],
                	'path'   => $this->config['cookie_path'],
                	'prefix' => $this->config['cookie_prefix'],
               );

		set_cookie($cookie);
	}
	
	
	/** 
	 *  Reads a cookie and parses the value to create a session. checks IP address 
	 *  against session table to verify session is valid.
	 *
	 *  @access	public
	 *  	
	 *  @return	false - if cookie not found or cookie is invalid
	 *              true  -  set current session
	 *
	 */
	function check_cookie_user()
	{
        $this->ci->load->helper('cookie');

		log_message('debug', 'MAUTH:	Checking Cookie User');

		$cookie = get_cookie( $this->config['cookie_prefix'] . $this->session_cookie_name, TRUE);
		
		if( ! $cookie ) return false;
		
		$user = unserialize($cookie);

		if(is_array($user))
		{
            //
			// TODO: possibly move these queries to a model
            //       password should be pre-salted...
            //
			$sql = "SELECT userid FROM ". $this->config['table_prefix'] ."users 
				WHERE username = '". $user['username'] ."' AND password ='". $user['password'] ."' 
				LIMIT 1";
			
			$res = $this->ci->db->query($sql);
			
			$row = $res->row_array();
			
			if( ! empty($row) )
			{
                $this->ci->load->add_package_path(APPPATH.'third_party/mauth/');
                $this->ci->load->model('users_model');

				$userid = $row['userid'];
				log_message('debug', 'MAUTH:	Logging In...');
				
				$user = $this->ci->users_model->get_user($user['username']);
					
				$this->ci->session->set_userdata( $user );
				$this->ci->session->set_userdata('logged_in', TRUE);
				$this->ci->session->set_userdata('strikes', 0); // reset strikes
			}
		}
		
		return false;
	}
	

	/** 
	 *  Registers a user to the temp registration table.  Returns and a 
     *  registration code used to complete registration.
	 *
	 *  @access	public
	 *  @param 	string 	username
	 *  @param	string 	email
	 *  @return 	string	Registration Code 
	 */
	function register($username, $email, $roleid = 0)
	{
		$regcode = md5($email . $username);
		
		if($roleid == 0)
			$roleid = $this->config['user_roleid'];
		
		$sql = "INSERT INTO ". $this->config['table_prefix'] ."users_reg 
					(username, email, roleid, regcode) 
				VALUES 
					('". $username ."',".$this->ci->db->escape($email).",". $roleid .",".$this->ci->db->escape($regcode).")";
		
		
		$this->ci->db->query($sql);
		
		if($this->ci->db->insert_id())
		{
			return $regcode;
		}
		return FALSE;
	}
	
	
	/** 
	 *  Confirm Registration
	 *	
	 *  Pass a registration code to this function and it will return a temp user
	 *  you should save a user with a password / passconfig combo when returned.
	 *
	 *  @access	public
	 *  @param	string	Registration Code
	 *  @return 	array	User array
	 *  @return	bool	error 
	 */
	function confirmRegistration($regcode = '')
	{
	
		if(empty($regcode)) return FALSE;
		
		$sql = "SELECT * FROM ". $this->config['table_prefix'] ."users_reg 
			WHERE regcode = ". $this->ci->db->escape($regcode). " LIMIT 1";
		
		$res = $this->ci->db->query($sql);
		
		$user = $res->row_array();
		
		return $user;
	}
	
	/** 
	 *  Removes a temp user from the temp user table
	 *
	 *  @access:	public
	 *  @param:	Reg code
	 *  @return bool;
	 */
	function delete_temp_user($regcode = '')
	{
		
		if(empty($regcode)) return FALSE;
		
		$sql = "DELETE FROM ". $this->config['table_prefix'] ."users_reg 
			    WHERE regcode = ". $this->ci->db->escape($regcode). " LIMIT 1";

		$res = $this->ci->db->query($sql);
			
		return $res;	
	}


	/**
 	 *  Make Password 
     *  creates a hashed password
	 * 
	 *  @access public
	 *  @param	string  password	     (plain text)
	 *  @return string  hashed password
	 */
	function make_password($password)
	{
  		$seed = substr('00' . dechex(mt_rand()), -3) .
    	
    		substr('00' . dechex(mt_rand()), -3) .
    		substr('0' . dechex(mt_rand()), -2);
  		
  		return $this->_hmac($seed, $password, 'md5', 64) . $seed;

	}
	
	/** 
	 *  Checks a password against a hashed password 
	 *  
	 *  @access private
	 *  @param	string password	     (plain text)
	 *  @param	string stored_value  (hashed)
	 *  @return	bool	
	 */
	function _check_password($password, $stored_value)
	{
  		$seed = substr($stored_value, 32, 8);
  		return $this->_hmac($seed, $password, 'md5', 64) . $seed==$stored_value;
	}

	/** 
	 *  HMAC (private)
	 *  Calculate HMAC according to RFC2104
	 *  http://www.ietf.org/rfc/rfc2104.txt
	 *
	 *  @access: private
	 *  @param:  string 	$key   	(seed)
	 *  @param:  string 	$data  	(the password) 
	 *  @param:  string 	$hash  	(hashing alg)
	 *  @param:  int    	$blocksize	size of string
	 *  @return: string	hashed password
	 * 
	 */
	function _hmac($key, $data, $hash = 'md5', $blocksize = 64)
	{
  		
  		if (strlen($key) > $blocksize)
		{
    			$key = pack('H*', $hash($key));
  		}
  		
  		$key  = str_pad($key, $blocksize, chr(0));
  		$ipad = str_repeat(chr(0x36), $blocksize);
  		$opad = str_repeat(chr(0x5c), $blocksize);
  
  		return $hash(($key^$opad) . pack('H*', $hash(($key^$ipad) . $data)));
	}

	
	/** 
	 *  Destroys a session, resets strikes, loggedin = 0
	 *
	 *  @access	public
	 *  @return 	bool
	 */  	
	function logout()
	{
        $this->ci->load->helper('cookie');

        $this->ci->session->set_userdata( array(
				    'username'	=> 'guest',
				    'userid'	=> 0,
				    'fbuserid'	=> 0,
				    'roleid'	=> 0,
				    'logged_in'	=> FALSE,
				    'email'		=> '',
				    'strikes'	=> 0,
				    'avatar'	=> '',
			        'fb_user'	=> FALSE
			    )
            );

        delete_cookie($this->config['cookie_prefix'] . $this->session_cookie_name);

		return TRUE;
	}
	
	
	/** 
	 *  Checks to see if a user is logged in.
	 *
	 *  @access	public
	 *  @return	bool
	 */
	function loggedin()
	{
		return $this->ci->session->userdata('logged_in');
	}
	
	
	/** 
	 *  A simple wrapper for getting the number of strikes
	 *
	 *  @access	public	
	 *  @return 	integer
	 */
	function strike_count()
	{
		return $this->ci->session->userdata('strikes');
	}
	
	
	/** 
	 *  Returns the session username
	 *	
	 *  @access	public
	 *  @return	string	username
	 */
	function get_username()
	{
		return $this->ci->session->userdata('username');
	}

	
	/** 
	 *  Gets the session users Email Address
	 *
	 *  @access	public
	 *  @return	string	
	 */
	function getEmail()
	{
		return $this->ci->session->userdata('email');
	}


	/** 
	 *  Is the session logged in via facebook
	 *
	 *  @access	public
	 *  @return	bool
	 */
	function fb_logged_in()
	{
        $fbuserid = $this->ci->session->userdata('fbuserid');
		if($fbuserid > 0) return TRUE;
		return FALSE;
	}


	/** 
 	 *  Checks to see if the current version matches db version.  If no version table exists,
	 *  we have a fresh install.  this function could provide xmlrpc lookups to see if software
	 *  is licened.
	 *
	 *  @access	private
	 *  @return	possible redirect if scripts version doesn't match the database version
 	 */
	function _check_version()
	{
		
		if( ! $this->ci->db->table_exists($this->config['table_prefix'] . 'store') )
		{
            log_message('debug', "\t\tMAUTH:\t!!!\tDatabase Installation needed.\t!!!");
			redirect('install', 301);
		} 
		else
		{
  			$res = $this->ci->db->query('SELECT * FROM `'. $this->config['table_prefix'] .'store` WHERE `key` LIKE \'version\'');
  			$ver = $res->row_array();
  			
  			if( ! defined('CURRENT_VERSION') )
			{
  				define('CURRENT_VERSION', $ver['data']);
  			}
  			
  			if(CURRENT_VERSION < MAUTH_VERSION)
			{
  					redirect('install');
  			}
  		}
	}

	
}

/* End of file mauth.php */ 
/* Location: ./application/libraries/mauth.php */ 
