<?php if ( !defined('BASEPATH') ) die('No direct script access allowed');

/**-------------------------------------------------------------------------
 *
 *	Permissions Model
 *	
 *	Provides methods for access restriction and querying the model.
 *
 * @file:  		application/third_party/mauth/models/permissions.php
 * @version:	0.9.4
 * @modified:	November, 2010
 * @author:		Pat Andrew	(http://patandrew.com/)
 * @link:		http://mauth.missionsix.net/
 *
 *-------------------------------------------------------------------------*/


class Permissions extends CI_Model
{
 	
 	// permissions store,  only accessible through php 5's __get()
 	private $perms = array();
 	
 	private $conf;
 	
 	/**
 	 *
 	 *	Construct 
 	 *	loads the current permissions for a user's role.
 	 *
 	 */
 	public function __construct()
	{
 		parent::__construct();

		if( isset($this->config) )
	 		$this->conf = $this->config->item('mauth');
 		else
			die('Cannot access CI Config');
	
		if( isset($this->session) )
	 		$this->load_permissions( $this->session->userdata('roleid') );
 		else
			die('CI Session Object not loaded');
 	}

	
 	/** 
 	 *	Public Check
 	 *	users permission properties to check if a user can do something.
 	 *
 	 *	@pre:	permission value is passed, eg  'permission_name'	
 	 *	@post:	Returns a boolean indicating if the user has the permission
 	 */
 	public function check($permission = '') {
 	
 		if(empty($permission)) return FALSE;
 		
 		$can = null;
 		
 		// recursive
 		if(is_array($permission)) {
 			foreach($permission as $perm) {
 				$can = $this->check($perm);
 			}
 		} else {
 			$can = $this->$permission; // using PHP5's __get()
 		}
 		
 		return $can;
 	}
 	
 	
 	/** 
 	 *	Public Verify Route
 	 *	uses roles & routes to verify user access granted in role_uri table.
 	 *	routes in table are absolute, meaning they rely on rsegments.
 	 *
 	 *	@post:	Route is either continued, or user is redirected to login/denied accordingly
 	 *
 	 */
 	public function verify_route()
    {
	 	$canroute =  $this->get_role_uri( $this->session->userdata('roleid') );
 	
 		if($canroute < 2) {
 			if($canroute == 0) {
                log_message('debug', 'User denied access. IP: '. $this->mauth->ip_address);
 				redirect( denied_url() );
 			}
 		}
 	}
 	
 	/**
 	 *	Get Role
 	 *	a role uri is grabbed from the db, if its there, we assume access is restricted for everyone but that role
 	 *	
 	 *	@post:	a value index is returned (0..2) 
 	 *			2 = don't care
 	 *			1 = valid
 	 *			0 = restricted
 	 */
 	public function get_role_uri($roleid = 0)
    {
 		
 		$uri_segs = $this->uri->rsegments; 
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		$segments = $this->uri->segment_array();
 		$uris = '';
 		if(!empty($segments)) {
			foreach($segments as $key=>$segment)
			{
				// $uris .= " XOR uri ='". substr($this->uri->uri_string, 1) ."'";
  			 	 $uris .= " OR uri ='". implode( '/', array_slice($segments,0,$key)) ."'";
			} 
		}
		
 	 	$valid = 2;
			 		
	 	// get uri_role permission for current URI.
	 	
	 	$sql = "SELECT * FROM {$table_prefix}role_uri 
	 			WHERE (uri = '". $uri_segs[1] ."'{$uris})";
	 		
	 	$res = $this->db->query($sql);
	 	
	 	$roles = $res->result_array();
	 		
	 	if($roles) { 			
	 		$valid = 0;
	 		$checkparentrole = TRUE;
	 		
	 		foreach($roles as $role) {
	 			if($role['roleid'] == $roleid) {
	 				$valid = $role['valid'];
	 				$checkparentrole = FALSE;
	 			}
	 		}	
	 		 		
	 		if($checkparentrole) {
                log_message('debug', 'MAUTH:	checking parent roleid');

		 		// get role parents
		 		$sql = "SELECT parentid FROM {$table_prefix}roles WHERE roleid={$roleid}";
		 		
		 		$res = $this->db->query($sql);
 	
		 		$parent = $res->row_array();
		 		
		 		if($parent) {
		 			if($parent['parentid'] > 0) {
		 				$valid = $this->get_role_uri($parent['parentid']);			
		 			}
		 		}
	 		
	 		}	
	 	}

	 	return $valid;
	 }


 	/** 
	 *	Get Role
	 *
	 *	@pre:	(int) $roleid
	 *	@post:	returns a roles details	
	 */
    public function get_role($roleid = 0)
    {
		if($roleid < 1) return FALSE;
		
		$table_prefix = $this->conf['table_prefix'];
		
		$sql = "SELECT * FROM {$table_prefix}roles WHERE roleid = $roleid LIMIT 1";
		$res = $this->db->query($sql);

		if($res) {
			return $res->row_array(); 
		}
		return FALSE;
	 }
	 
	/**
 	 * Get Permission
	 * returns all details related to a permission
	 */
	public function get_permission( $permid = 0 )
	{
		if( $permid == 0 ) return FALSE;

		$perms =  $this->get_permissions($permid);

		if( !empty($perms) )
			return $perms[0];

		return FALSE;
	}
	 
 	/** 
 	 *	Load Permissions
 	 *	loads $this->data with permissions for the current logged in user.
 	 *
 	 */
 	private function load_permissions($roleid = 0)
    {
        log_message('debug', "MAUTH:	Loading Permissions for role #$roleid");	 	

 		// if($roleid < 1) return FALSE;
 		
 		$perms = $this->get_role_permission($roleid);
 		
 		if($perms) {	
			foreach($perms as $k => $v) {	
 				$this->$k = (bool) $v;  //uses PHP5 __set();
 			}
 		}
 	}
 	
 	/** 
 	 *	Get Role Permissions
 	 *	Returns an array of permissions for a given role ID.   
 	 *  If a role has a parent, a recursive call to getPermissions() is performed
 	 *	which is the base permissions for the current role id.  
 	 *
 	 * 	@params: roleid
 	 *	@return:  Array indexed by permissions names of BOOLEANs.
 	 *				ex:		$perms['CAN_EDIT_STYLES'] = TRUE;
 	 */
 	
 	public function get_role_permission($roleid = 0)
    {
 		
 		if($roleid == 0) return FALSE;
 
		log_message('debug', "MAUTH:	Getting Permission for role $roleid");	 	
 		
 		// permissions
 		$perms = array();
 		$parentperms = array();
 		
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		// get role parents
 		$sql = "SELECT parentid FROM {$table_prefix}roles WHERE roleid={$roleid}";
 		$res = $this->db->query($sql);
 		
 		$parent = $res->row_array();
 		
 		if(!$parent) return FALSE;
 		
 		if(!empty($parent)) {
	 		if($parent['parentid'] != 0) {
	 			$parentperms = $this->get_role_permission($parent['parentid']);
	 			if(is_array($parentperms)) {
		 			// overload & maintain previous permissions.
		 			foreach($parentperms as $k => $v) {
		 				$perms[$k] = $v;
		 			}
	 			}
				else
				{
	 				die('Failed to properly load parent permissions.');
	 			}
 			}
 		}
 		
 		// get permissions.

 		$sql = "SELECT {$table_prefix}permissions.*, {$table_prefix}role_permissions.value, 
						{$table_prefix}role_permissions.uri
 				FROM {$table_prefix}role_permissions
 				LEFT JOIN {$table_prefix}permissions 
					ON ({$table_prefix}permissions.permid = {$table_prefix}role_permissions.permid) 
 				WHERE {$table_prefix}role_permissions.roleid = $roleid";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		$results = $res->result_array();
 		
 		if(!empty($results)) {
 		
 			foreach($results AS $permission) {
 				$perms['CAN_'. $permission['title']] = $permission['value']; 
 			}
 		}
 		
 		return $perms;
 	}
 	
 	
 	
 	/**
 	 *	Permissions
 	 *	returns an array of all permissions with sub-arrays of rolenames & roleids
 	 *
 	 */
 	public function get_permissions( $permid = 0 )
	{
 		
 		$table_prefix = $this->conf['table_prefix'];
 		
 		$sql = "SELECT {$table_prefix}permissions.*, {$table_prefix}role_permissions.value, {$table_prefix}role_permissions.uri, 
	 GROUP_CONCAT({$table_prefix}roles.name SEPARATOR ',') AS rolenames,
	 GROUP_CONCAT({$table_prefix}roles.roleid SEPARATOR ',') AS roleids,
	 GROUP_CONCAT({$table_prefix}role_permissions.value SEPARATOR ',') AS rolevalues
	 FROM {$table_prefix}role_permissions
	 LEFT JOIN {$table_prefix}permissions ON ({$table_prefix}permissions.permid = {$table_prefix}role_permissions.permid)
	 INNER JOIN {$table_prefix}roles ON ({$table_prefix}roles.roleid = {$table_prefix}role_permissions.roleid)
	 ". ( $permid > 0 ? "WHERE {$table_prefix}permissions.permid = {$permid} " : '') ."
	 GROUP BY {$table_prefix}permissions.permid
	";

 		$res = $this->db->query($sql);
 		
 		if( $res->num_rows() < 1 ) return array();

 		$permissions = $res->result_array();
 		
 		foreach($permissions as $key => $permission)
		{
	 		$permissions[$key]['rolenames'] = explode(',', $permission['rolenames']);
	 		$permissions[$key]['roleids'] = explode(',', $permission['roleids']);
            $permissions[$key]['rolevalues'] = explode(',', $permission['rolevalues']);
	 	}	 	
 		return $permissions;
 	}
 	
 	/** 
 	 *	Get Roles
 	 *	returns an array of roles / titles 
 	 *
 	 */
 	 public function get_roles() {
 	 
 	 	$table_prefix = $this->conf['table_prefix'];
 		
 		$sql = "SELECT {$table_prefix}roles.* FROM {$table_prefix}roles";
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		return $res->result_array();
 	 }
 	  	
 	
 	/**
 	 *	Save Permissions
 	 *	allows you to save permissions to the database
 	 *
 	 *	@pre:	permission title is defined, description can be empty;
 	 *
 	 *	@post:	permission is saved in db. insert id is returned
 	 */
 	public function save_permission($title = '', $description = '', $permid = 0)
	{
 		
 		if(empty($title)) return FALSE;
 	
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "REPLACE INTO {$table_prefix}permissions (".($permid > 0 ? 'permid,': '')."title, description) 
				VALUES (".($permid > 0 ? $permid .',': ''). $this->db->escape($title) .",". $this->db->escape($description). ')';
 		
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		return $this->db->insert_id();
 		
 	}
 	
 	/** 
 	 *	Save Permission Role Assoc
 	 *	pass a roleid, permid and value 
 	 */
 	public function save_role_perm_assoc($roleid = 0, $permid = 0, $value = 0, $uri = '')
	{
 		// REPLACE INTO doesn't work here, since we don't have a primary key.
		// we have to forcefully remove the old entr(y|ies)
		// FIRST, we grab the entry for saving.

 		$table_prefix = $this->conf['table_prefix'];

		// preserve URI if not set previously
		if( $uri == '' ) {
			$sql = "SELECT uri FROM {$table_prefix}role_permissions WHERE roleid={$roleid} AND permid={$permid} LIMIT 1";
 			$res = $this->db->query($sql);
 			$res = $res->row_array();
 			if( ! empty($res) )
				$uri = $res['uri'];
		}

		$this->delete_role_perm_assoc($roleid, $permid);

 		$sql = "INSERT INTO {$table_prefix}role_permissions (roleid, permid, value, uri)
                VALUES ({$roleid},{$permid},{$value},". $this->db->escape($uri) .")";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;

		// save URI permission
		$this->save_uri_permission($uri, $roleid, $value);
 		
 		return TRUE;
 	}
 	

	/**
	 * Delete role permission association
	 */
 	public function delete_role_perm_assoc($roleid = 0, $permid = 0)
	{
		$table_prefix = $this->conf['table_prefix'];
 		$sql = "DELETE FROM {$table_prefix}role_permissions 
				WHERE roleid=". $this->db->escape($roleid) ." AND permid = ". $this->db->escape($permid);
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		return TRUE;
	}


 	/** 
 	 *	Save URI Permission
 	 *	
 	 *	@pre:	permid, uri, value are defined
 	 *	@post:	saved in db.
 	 *
 	 */
 	public function save_uri_permission($uri = '', $roleid = 0, $value = 0)
	{
 	
		$this->delete_uri_role_assoc($uri, $roleid);

 		if(empty($uri)) return FALSE;
 		
 		$table_prefix = $this->conf['table_prefix'];
 		$sql = "INSERT INTO {$table_prefix}role_uri (roleid, uri, valid) 
				VALUES ({$roleid},". $this->db->escape($uri) .",{$value})";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		return TRUE;
 	}


	/**
	 * Delete URI role assoc
	 */
 	public function delete_uri_role_assoc($uri ='', $roleid = 0) {

		if(empty($uri)) return FALSE;

 		$table_prefix = $this->conf['table_prefix'];	
		/*
			$sql = "SELECT value FROM {$table_prefix}role_uri WHERE uri=". $this->db->escape($uri) ." LIMIT 1";

			$res = $this->db->query($sql);

			if( !empty($res = $res->result_array()) )
			{
				if( $res['value'] == 0 )   // don't delete the role entry  if its a denied restriction.
					return TRUE;
			}
		*/
 		$sql = "DELETE FROM {$table_prefix}role_uri 
				WHERE roleid = {$roleid} AND uri =". $this->db->escape($uri) ;
 	
 		$res = $this->db->query($sql);
		
		if(!$res) return FALSE;

		return TRUE;
	}

 	
 	/** 
 	 *	Add Roles
 	 *	add a new role to the db
 	 */
 	public function save_role($name = '', $parentid = 0, $roleid = 0)
	{
 		
 		if(empty($name)) return FALSE;
 		
 		$table_prefix = $this->conf['table_prefix'];

		if( $roleid > 0 )
			$sql = "REPLACE INTO {$table_prefix}roles 
						(roleid, name, parentid) 
					VALUES 
						({$roleid}, ". $this->db->escape($name) .", {$parentid})";
 		else
	 		$sql = "INSERT INTO {$table_prefix}roles (name, parentid)
					VALUES (". $this->db->escape($name) .", {$parentid})";
 	
 		$res = $this->db->query($sql);
 		
 		if(!$res) return FALSE;
 		
 		return TRUE;
 		
 	}
 	


 	/**
 	 *	Getter
 	 *	provides access to $data
 	 *  must provide CAN_  in front of permission name.
 	 *
 	 *	use:   		$permissions->VAR
 	 *	returns:	$this->data[VAR] OR FALSE.
 	 */
	public function __get($key) {
	     
		$key = $this->_strip_can($key);

	    if ( array_key_exists($key, $this->perms) ) {
			return $this->perms[$key];
		} else {
			$CI =& get_instance();
			return ( isset($CI->$key)? $CI->$key : NULL);
        }
	}

 	
 	/**
 	 *	Permissions Setter
 	 *	provides write access to $data
 	 *  must provide CAN_  in front of permission name.
 	 *
 	 *	use:   		$permissions->VAR
 	 *	returns:	$this->data[VAR] OR FALSE.
 	 */ 	
 	public function __set($name, $value = TRUE)
	{
		$name = $this->_strip_can($name);

		$this->perms[$name] = (bool) $value;

		return $value;
 	}
 	

    public function __isset($key) {
    	$key = $this->_strip_can($key);
        if(array_key_exists($key , $this->perms) ) 
			return TRUE;
		else {
			$CI =& get_instance();
			return isset($CI->$key);
		}	
    }

    
    public function __unset($name) {
        $name = $this->_strip_can($name);
        unset($this->data[$name]);
    }
    
    
    
    /** 
     *	Private Function Strip Can
     *	removes CAN_ from permission name
     *  @return: String;
     */
    private function _strip_can($name) {
    	$prefix = substr($name, 0,4);
    	
    	if($prefix == 'CAN_') { 
    		return strtolower(substr($name, 4));
    	}
    	return strtolower($name);
    }
 }

/* End of file permissions.php */ 
/* Location: ./application/models/permissions.php */ 
